//  BRACES is a simple brace count program.  It tries to detect
//  mismatched braces.  Right now the function is very simple:
//  It will detect mismatched (, [, and {, and it ignores
//  text in C or C++ style comments and it ignores strings.
//
//  BRACES is implemented as a state machine.
//
//  Written by Brad B. Brown, Information Engineering, 1991
//  This program has been released to the public domain with
//  no restrictions on the duplication or use of source code.


#include <stdio.h>
#include <stdlib.h>
#include <dir.h>



void processFile( FILE * input )
{
      int *       chars;
      int         state;
      int         c;
      int         deltaBraces;
      int         deltaParens;
      int         deltaBrackets;


      chars = (int *)calloc( 128, sizeof( int ) );
      if ( chars == (int *)NULL ) {
            fputs( "Out of memory.\n", stdout );
            return;
            }


      c = 0;
      state = 0;
      while ( c != (int)EOF ) {

            c = fgetc( input );

            switch ( state ) {
              
                  case 0:
                        if ( c == (int)'/' )
                              state = 1;
                        else
                              if ( c == (int)'\"' )
                                    state = 3;
                              else
                                    if ( c == (int)'\'' )
                                          state = 5;
                                    else
                                          if ( c < 128 )
                                                chars[c]++;
                        break;

                  case 1:
                        if ( c == (int)'/' )
                              state = 2;
                        else if ( c == (int)'*' )
                              state = 7;
                        else
                              state = 1;
                        break;

                  case 2:
                        if ( c == (int)'\n' )
                              state = 0;
                        break;

                  case 3:
                        if ( c == (int)'\\' )
                              state = 4;
                        else
                              if ( c == (int)'\"' )
                                    state = 0;
                        break;

                  case 4:
                        state = 3;
                        break;

                  case 5:
                        if ( c == (int)'\\' )
                              state = 6;
                        else
                              if ( c == (int)'\'' )
                                    state = 0;
                        break;

                  case 6:
                        state = 5;
                        break;

                  case 7:
                        if ( c == (int)'*' )
                              state = 8;
                        break;

                  case 8:
                        if ( c == (int)'/' )
                              state = 0;
                        else
                              state = 7;
                        break;
                  }
            }


      //  Now we have reached the end of the file.  Close the file
      //  and report the results.

      fclose( input );

      deltaBraces   = chars['{'] - chars['}'];
      deltaParens   = chars['('] - chars[')'];
      deltaBrackets = chars['['] - chars[']'];

      if ( deltaBraces != 0 )
            if ( deltaBraces > 0 )
                  printf( "Missing %d '}'.  ", deltaBraces );
            else
                  printf( "Missing %d '{'.  ", - deltaBraces );

      if ( deltaParens != 0 )
            if ( deltaParens > 0 )
                  printf( "Missing %d ')'.  ", deltaParens );
            else
                  printf( "Missing %d '('.  ", - deltaParens );

      if ( deltaBrackets != 0 )
            if ( deltaBrackets > 0 )
                  printf( "Missing %d ']'.  ", deltaBrackets );
            else
                  printf( "Missing %d '['.  ", - deltaBrackets );

      fputs( "\n", stdout );
}


void doFile( char * name )
{
      FILE *      input;


      printf( "%13s: ", name );

      input = fopen( name, "r" );
      if ( input == (FILE *)NULL ) {
            fputs( "Can't open\n", stdout );
            }
      else
            processFile( input );

      fclose( input );
}



int main( int argc, char ** argv )
{
      for ( int arg = 1;  arg < argc;  arg++ ) {

            struct ffblk      fblk;


            if ( findfirst( argv[arg], & fblk, 0 ) == 0 ) {

                  doFile( fblk.ff_name );
                  while ( findnext( & fblk ) == 0 )
                        doFile( fblk.ff_name );

                  }

            }

      return 0;
}
